<?php
namespace chat\system\command;
use chat\data\message\MessageAction;
use chat\data\room\Room;
use wcf\data\user\UserProfile;
use wcf\system\exception\PermissionDeniedException;
use wcf\system\WCF;

/**
 * dice command
 *
 * @author		Daniel (Keito) Lang
 * @copyright	2014-2019 da-lang.net
 * @package		net.dalang.chat.command.dice
 */
class DiceCommand extends AbstractCommand implements ICommand {
	/**
	 * dice number
	 * @var integer
	 */
	public $dice = 1;
	
	/**
	 * value number
	 * @var integer
	 */
	public $value = 1;
	
	/**
	 * @inheritdoc
	 */
	public function getJavaScriptModuleName() {
		return 'Dalang.net/Chat/Command/Dice';
	}
	
	/**
	 * @inheritdoc
	 */
	public function validate($parameters, Room $room, UserProfile $user = null) {
		if ($user === null) $user = new UserProfile(WCF::getUser());
		
		if (!$room->canWritePublicly($user)) {
			throw new PermissionDeniedException();
		}
		
		if (!$user->getPermission('user.chat.canUseDice')) {
			throw new PermissionDeniedException();
		}
		
		$this->dice = $this->assertParameter($parameters, 'dice');
		$this->value = $this->assertParameter($parameters, 'value');
		
		// validate max dice number
		if ($this->dice < 1) $this->dice = 1;
		if ($this->dice > CHAT_DICE_MAXDICE) $this->dice = CHAT_DICE_MAXDICE;
		
		// validate max dice value
		if ($this->value < 1) $this->value = 1;
		if ($this->value > CHAT_DICE_MAXVALUE) $this->value = CHAT_DICE_MAXVALUE;
	}
	
	/**
	 * @inheritdoc
	 */
	public function execute($parameters, Room $room, UserProfile $user = null) {
		if ($user === null) $user = new UserProfile(WCF::getUser());
		
		// throw the dice, baby!
		$values = [];
		for ($i = 1; $i <= $this->dice; $i++) {
			$values[] = mt_rand(1, $this->value);
		}
		$rollString = ($this->dice > 20 ? '' : implode(' + ', $values));
		$totalValue = array_sum($values);
		
		$action = new MessageAction([], 'create', [
				'data' => [
						'roomID' => $room->roomID,
						'userID' => $user->userID,
						'username' => $user->username,
						'time' => TIME_NOW,
						'objectTypeID' => $this->getMessageObjectTypeID('net.dalang.chat.messageType.dice'),
						'payload' => serialize(['dice' => $this->dice, 'value' => $this->value, 'rollString' => $rollString, 'totalValue' => $totalValue])
				],
				'updateTimestamp' => true,
				'grantPoints' => false
		]);
		$action->executeAction();
	}
}